% Induction motor running under pulsating loads
% Choice of reference frame
% Currents as state variables
% per unit formulation

  close all,clc
  
 info= ...
        ['                             INDUCTION MOTOR RUN                                '                                
         '                    (Currents and speed as state variables)                     '
         ' The motor is running at a specified load. After reaching steady state in an    '   
         ' initialization first run, the initial conditions are established for a second  '
         ' run in which the load is pulsating.                                            ' 
         ' The dynamic equations are expressed in canonical first-order matrix form       ' 
         ' adopting currents and speed as state variables, and are solved using ODE45.    '
         ' The user has the choice of selecting one of three common (dq) reference frames:'
         ' STATOR, ROTOR or SYNCHRONOUS.                                                  '
         ' The torque, speed,and current responses are plotted.                           '
         ' -------------------------------------------------------------------------------'];                                
      disp(info)
 
 % -------------------------  MODE SELECTION  ----------------------------
      kr=menu('Select operating mode','INITIALIZATION','RUN','QUIT');
                if kr==1,  init=1;                  
                   elseif kr==2,  init=2;          
                   else return;
                end    
 % --------------------     MACHINE PARAMETERS [pu] -----------------
Vs=1.0   ; %Stator voltage 
Rs=0.01  ; %Stator resistance 
Rr=0.02  ; %Rotor resistance  
Lsl=.10 ;  %Stator leakage inductance
Lrl=.10  ; %Rotor leakage inductance  
Lm=4.5  ;  %Magnetizing inductance  
ws=1.0;    %Stator frequency
freq0=60 ; %Base frequency (Hz)
wo=2*pi*freq0 ; %Base frequency (rad/s)
H=.30    ; % Inertia constant (s)
TLi=.1;    %Initial load torque 
TLf=.8;    %Final load torque 
Bm=1e-6;   %Frictional coefficient
% -------------------------INDUCTANCE MATRIX-------------------------
Ls=Lsl+Lm ; Lr=Lrl+Lm;

L=[Ls  0   Lm  0   
   0   Ls  0   Lm 
   Lm  0   Lr  0
   0   Lm  0   Lr];

 % --------------------  REFERENCE FRAME SELECTION  ----------------------------
      km=menu('Select a common reference frame','STATOR','SYNCHRONOUS','ROTOR');
            
 % ---------------------------  ODE  -------------------------------------------
 if init==1, xo=[0 0 0 0 ws 0]';         %initial conditions at initialization
   elseif init==2, xo=x(length(t),1:6)'; %initial conditions for rerun
 end    
 tf=2 ;                                  % final time
 tspan=[0 tf] ; 
 options=odeset('RelTol',1e-3,'maxstep',1e-3); %,'outputfcn','odeplot','outputsel',5) ;
 [t,x]=ode45(@IMcurf0,tspan,xo,options,Rs,Rr,Lm, Lsl,Lrl, Vs ,wo, km,H,Bm,TLi,TLf,init);
 % --------------------  OUTPUT VECTORS --------------------------------
 spd=x(:,5); 
 cur=x(:,1:4)';
 flux=L*cur;
 ids=x(:,1)  ;  iqs=x(:,2) ;
 idr=x(:,3)  ;  iqr=x(:,4) ;
 Te=Lm*(iqs.*idr-ids.*iqr);
 phids=flux(1,:)';phiqs=flux(2,:)';
  psi=x(:,6);
  vas=Vs*cos(wo*t); vbs=Vs*sin(wo*t);
  vds=vas.*cos(psi)+vbs.*sin(psi);  % Voltage expressed in dq common reference frame
  vqs=-vas.*sin(psi)+vbs.*cos(psi) ;  
  % --------------------  GRAPHICAL DISPLAYS  ----------------------------------
  figure('Units','Normalized','Position',[.02 .54 .5 .35],'Name','Induction motor transients');
  figure(1)
   plot(t,spd,'color',[0.5 0 0]);title('Speed vs time');xlabel('Time [s]'),ylabel('Speed [pu]'),grid
   figure('Units','Normalized','Position',[.02 .09 .5 .35],'Name','Induction motor transients');
  figure(2) 
   plot(0,0,'w',t,ids,'-r',t,iqs,'-g');title('Currents vs time','color','r');xlabel('Time [s]'),ylabel('ids & iqs [pu]'),grid
    if km==1, ref='Stator frame';
     elseif km==2, ref='Synchronous frame';
     else ref='rotor frame';
     end
  legend(ref)
  figure('Units','Normalized','Position',[.52 .54 .45 .35],'Name','Induction motor transients');
  figure(3)
   plot(t,Te);title('Torque vs time');xlabel('Time [s]'),ylabel('Torque [pu]'),grid
   figure('Units','Normalized','Position',[.52 .09 .45 .35],'Name','Induction motor transients');
  figure(4)
   plot(spd,Te);title('Torque vs Speed');xlabel('Speed [pu]'),ylabel('Torque [pu]'),grid
  figure('Units','Normalized','Position',[.35 .09 .4 .35],'Name','Induction motor transients'); 
  figure(5)
   plot(0,0,'w',t,phids,'-g',t,phiqs,'-y');title('Stator fluxes vs time');xlabel('Time [s]'),ylabel('phids & phiqs [pu]'),grid 
   if km==1, ref='Stator frame';
     elseif km==2, ref='Synchronous frame';
     else ref='rotor frame';
   end
   legend(ref)
   figure('Units','Normalized','Position',[.35 .54 .4 .35],...
          'Name','Induction motor transients','menubar','figure'); 
   figure(6);
   plot(t,vds,'-m',t,vqs,'-b');title('Voltage vs time','color','b');xlabel('Time [s]'),ylabel('Voltage [pu]'),grid 
     if km==1, ref='Stator frame';
     elseif km==2, ref='Synchronous frame';
     else ref='rotor frame';
     end
  legend(ref)
  
   h0=figure('Units','Normalized','menubar','none','numbertitle','off','Position',[.87 .8 .1 .15]); 
 
      uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','close all,clc,return', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.06 0.55 0.2], ...
	      'String','CLOSE', ...
	      'TooltipString','Close'); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit imcur0', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.38 0.55 0.2], ...
	      'String','EDIT', ...
	      'TooltipString','Edit');
      uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','run imcur0', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.7 0.55 0.2], ...
	      'String','RERUN', ...
	      'TooltipString','Run');

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
