% STATE EQUATIONS OF THE INDUCTION MACHINE
% Choice of reference frame
% Currents as state variables
% per unit formulation
% =========================================================================================
function IMmod1
   clc,clear
   date
   info= ...
        ['                             INDUCTION MOTOR START                              '                                
         '                             (per-unit formulation)                             '
         ' The purpose of this script file is to show how the state equations of an       '   
         ' induction machine are assembled in the context of a simple mode of operation:  '
         ' namely, online starting. The state variables are the machine currents ids, iqs,' 
         ' idr, iqr; the speed wm; and the position psi of the common reference frame.    ' 
         ' The user may select one of three common (dq) reference frames: STATOR, ROTOR,  '
         ' or SYNCHRONOUS. The dynamic equations are solved using ODE45.                  '
         ' The torque, speed,and current responses are plotted.                           '
         ' To run this program, enter the filename <<< IMmod1 >>>                         '
         ' To run the corresponding steady-state characteristics, enter the filename      '  
         ' <<< IMstatic1 >>> or click the << STEADY STATE >> button.                      '
         ' -------------------------------------------------------------------------------'];                                
    disp(info)
 
 % --------------------  REFERENCE FRAME SELECTION  ------------------------------------              
 km=menu('Select a common reference frame','STATOR','SYNCHRONOUS','ROTOR','QUIT');
           if km==4,clc,close all;return; end
 % ---------------------------  ODE  ---------------------------------             
 xo=[0 0 0 0 0 0]';                     % initial conditions 
 tf=2.5 ;                               % final time (s)
 Lm=4.50;                               % magnetizing inductance [pu]
 tspan=[0 tf] ; 
 options=odeset('RelTol',1e-3,'maxstep',4e-4); 
 [t,x]=ode45(@IMstate1,tspan,xo,options,km,Lm) ;   
 % --------------------  OUTPUT VARIABLES  ---------------------------     
 spd=x(:,5);                 % Speed
 ids=x(:,1)  ;  iqs=x(:,2) ; % Stator...
 im=sqrt(ids.*ids+iqs.*iqs); %          currents
 idr=x(:,3)  ;  iqr=x(:,4) ; % Rotor currents
 Te=Lm*(iqs.*idr-ids.*iqr) ; % Electromagnetic torque
 % ------------------------- PLOTS ------------------------------------
       IMgraf1
 %     edit IMgraf1 to view plot instructions
% ========================================================================================

function dxdt=IMstate1(t,x,km,Lm)         
% --------------------     MACHINE PARAMETERS [pu] --------------------
Vs=1.0   ; % Stator voltage 
ws=1.0;    % Stator frequency
freq0=60 ; % Base frequency (Hz)
wo=2*pi*freq0 ; % Base frequency (rad/s)
Rs=0.01  ; % Stator resistance 
Rr=0.02  ; % Rotor resistance  
Lsl=.10 ;  % Stator leakage inductance
Lrl=.10  ; % Rotor leakage inductance  
H=0.60 ;   % Inertia constant (s)
Bm=1e-3;   % Frictional coefficient
TLi=.01;   % Initial load torque 
TLf=1.2;   % Final load torque 
t1=2.0 ;   % Time for load application (s)
Ls=Lsl+Lm ; Lr=Lrl+Lm;
% -------------------- REFERENCE FRAME -------------------------------      
  wm=x(5) ;          % rotor speed
  psi=x(6);          % reference frame position with respect to stator
  
      if km==1, wk=0;              % STATOR
       elseif km==2,  wk=ws;       % SYNCHRONOUS
       elseif km==3,  wk=wm;       % ROTOR
      end
% ---------------------- INPUTS ---------------------------------------     
  %  Load time profile :    
       if t<t1, TL=TLi; else TL=TLf; end    
  %  Stator voltage time profile  : 
  %   Stator 2-phase voltage excitation in stator frame:
       gam=pi/3;
       vas=Vs*cos(ws*wo*t+gam);    vbs=Vs*sin(ws*wo*t+gam); 
% ------------------------ STATE EQUATIONS -----------------------------
  % State  vector:       x=[ids iqs idr iqr wm psi]'
  % Stator voltage excitation in common reference frame:
       vds= vas*cos(psi)+vbs*sin(psi); vqs=-vas*sin(psi)+vbs*cos(psi); 
 
    V=[vds vqs 0 0]';                    % Voltage vector
    
    I=[x(1) x(2) x(3) x(4)]';            % Current vector
    
    R=diag([Rs Rs Rr Rr]);               % Resistance matrix   
       
    L=[Ls  0   Lm  0   
       0   Ls  0   Lm 
       Lm  0   Lr  0
       0   Lm  0   Lr];                  % Inductance matrix
    
    rot90=[0 -1;1 0];    O=zeros(2);        
    
    W=[wk*rot90       O                  % Reference frame speed matrix ...
          O    (wk-wm)*rot90];           % with respect to stator and rotor      
    
    Z=R+W*L;                             % Impedance (real) matrix  
      
    Te=Lm*(x(2)*x(3)-x(1)*x(4));         % Electromagnetic torque
    
  %%% Matrix formulation of induction machine equations in common reference frame| 
  %%% Constitutive flux-current relations:
  %%%    F = L*I   or  I = L\F  
  %%% Electrical system equations:
  %%%    V= R*I + W*F + dF/wodt = R*I + W*L*I +dF/wodt = (R + W*L)*I + L*dI/wodt  
  %%% Mechanical system equation:
  %%%    Te = 2*H*dwm/dt + TL + Bm*wm
    
   didt=L\(V-Z*I)*wo;                  % Electrical system equations
   dwmdt=(Te-TL)/(2*H)-Bm*wm/(2*H);    % Mechanical system equation
   
   dxdt=[didt;dwmdt;wk*wo];            % State equations obtained by concatenation
% ===================================================================================
