% Induction motor line starting
% Choice of reference frame
% Currents as state variables
 function IMstartcur
 close all, clc
 warning off
 global Lm L p Vs wo wk km
 info= ...
        ['                         INDUCTION MOTOR STARTING                               '                                
         '                    (Currents and speed as state variables)                     '
         ' The motor is starting at no load and, after reaching steady-state speed, a step'   
         ' load torque is suddenly applied.                                               '  
         ' The dynamic equations are expressed in canonical first-order matrix form       ' 
         ' adopting currents and speed as state variables, and are solved using ODE45.    '
         ' The machine parameters may be changed by editing the file <IMstartcur.m>.      ' 
         ' The user has the choice of selecting one of three common (dq) reference frames:'
         ' STATOR, ROTOR or SYNCHRONOUS.                                                  '
         ' The torque, speed,and current responses are plotted.                           '
         ' PAUSED: Press any key to view plots.                                           '];                                    
      disp(info)
 pause
 % --------------------  REFERENCE FRAME SELECTION  ----------------------------
      km=menu('Select a common reference frame','STATOR','SYNCHRONOUS','ROTOR','QUIT');
                
 % ---------------------------  ODE  -------------------------------------------
 xo=[0 0 0 0 0 0]';        %initial conditions 
 tf=2 ;               % final time
 tspan=[0 tf] ; 
 options=odeset('RelTol',1e-3,'maxstep',1e-3); %,'outputfcn','odeplot','outputsel',5) ;
 [t,x]=ode45(@IMstartf,tspan,xo,options);
 % --------------------  OUTPUT VARIABLES  --------------------------------------
 spd=x(:,5)*60/(p*pi);  %rpm
 cur=x(:,1:4)';
 flux=L*cur;
 ids=x(:,1)  ;  iqs=x(:,2) ;
 idr=x(:,3)  ;  iqr=x(:,4) ;
 Te=(3/2)*(p/2)*Lm*(iqs.*idr-ids.*iqr);
 phids=flux(1,:)';phiqs=flux(2,:)';
  psi=x(:,6);
  vas=Vs*cos(wo*t); vbs=Vs*sin(wo*t);
  vds=vas.*cos(psi)+vbs.*sin(psi);  % Voltage expressed in dq common reference frame
  % --------------------  GRAPHICAL DISPLAYS  ----------------------------------
  figure('Units','Normalized','Position',[.02 .54 .5 .35],'Name','Induction motor starting transients');
  figure(1)
   plot(t,spd,'color',[0.5 0 0]);title('Speed vs time');xlabel('Time [s]'),ylabel('Speed [rpm]'),grid
   figure('Units','Normalized','Position',[.02 .09 .5 .35],'Name','Induction motor starting transients');
  figure(2) 
   plot(0,0,'w',t,ids,'-r',t,iqs,'-g');title('Currents vs time','color','r');xlabel('Time [s]'),ylabel('ids & iqs [A]'),grid
    if wk==0, ref='Stator frame';
     elseif wk==wo, ref='Synchronous frame';
     else ref='rotor frame';
     end
  legend(ref)
  figure('Units','Normalized','Position',[.52 .54 .45 .35],'Name','Induction motor starting transients');
  figure(3)
   plot(t,Te);title('Torque vs time');xlabel('Time [s]'),ylabel('Torque [N.m]'),grid
   figure('Units','Normalized','Position',[.52 .09 .45 .35],'Name','Induction motor starting transients');
  figure(4)
   plot(spd,Te);title('Torque vs Speed');xlabel('Speed [rpm]'),ylabel('Torque [N.m]'),grid
  figure('Units','Normalized','Position',[.35 .09 .4 .35],'Name','Induction motor starting transients'); 
  figure(5)
   plot(0,0,'w',t,phids,'-g',t,phiqs,'-y');title('Stator fluxes vs time');xlabel('Time [s]'),ylabel('phids & phiqs [Wb]'),grid 
   if wk==0, ref='Stator frame';
     elseif wk==wo, ref='Synchronous frame';
     else ref='rotor frame';
   end
   legend(ref)
   figure('Units','Normalized','Position',[.35 .54 .4 .35],...
          'Name','Induction motor starting transients','menubar','figure'); 
   figure(6);
   plot(0,0,'w',t,vds,'-m');title('Voltage vs time','color','b');xlabel('Time [s]'),ylabel('Voltage [V]'),grid 
     if wk==0, ref='Stator frame';
     elseif wk==wo, ref='Synchronous frame';
     else ref='rotor frame';
     end
  legend(ref)
  
   h0=figure('Units','Normalized','menubar','none','numbertitle','off','Position',[.87 .8 .1 .15]); 
 
      uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','clc,clear,close all,return', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.06 0.55 0.2], ...
	      'String','CLOSE', ...
	      'TooltipString','Close'); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit imstartcur', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.4 0.55 0.2], ...
	      'String','EDIT', ...
	      'TooltipString','Edit');
      uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','run imstartcur', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.7 0.55 0.2], ...
	      'String','RUN', ...
	      'TooltipString','Run');

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dx=IMstartf(t,x)
global Lm L p Vs wo wk km
% --------------------     MACHINE PARAMETERS ----------------------
Vll=220    ; %Line-to-line rms voltage rating [V]
Rs=0.531    ; %Stator resistance (Ohm)
Rr=0.408   ; %Rotor resistance  (Ohm)
Lsl=2.5e-3 ; %Stator leakage inductance(H)
Lrl=2.5e-3    ; %Rotor leakage inductance  (H)
Lm=84.7e-3   ; %Magnetizing inductance  (H) 
freq0=60   ; %Base frequency (Hz)
wo=2*pi*freq0 ; %Base frequency (rad/s)
p=4        ; %number of poles
J=0.1    ; %Moment of inertia (kg.m^2)
TLf=10; %Load torque [N.m]
Bm=.01; %Frictional coefficient
% -------------------------INDUCTANCE MATRIX-------------------------
Ls=Lsl+Lm ; Lr=Lrl+Lm;

L=[Ls  0   Lm  0   
   0   Ls  0   Lm 
   Lm  0   Lr  0
   0   Lm  0   Lr];
        
% --------------------------STATE EQUATIONS-------------------------- 
  wm=x(5) ;
               if km==1,  wk=0;                  
                   elseif km==2,  wk=wo;          
                   elseif km<=3,  wk=wm;
                end
        %if km==3, wk=wm; end
        if t<0.7, TL=0;else TL=TLf; end
 Vs=Vll*sqrt(2/3);
 vas=Vs*cos(wo*t);    vbs=Vs*sin(wo*t);
 
 vds=vas*cos(x(6))+vbs*sin(x(6)) ;
 vqs=-vas*sin(x(6))+vbs*cos(x(6)) ; 
 
    V=[vds vqs 0 0]';
    I=[x(1) x(2) x(3) x(4)]'; 
    
    R=diag([Rs Rs Rr Rr]);
   
    W=[ 0        -wk         0        0
      wk         0          0        0
       0         0          0     -(wk-wm)
       0         0       (wk-wm)     0     ];
   
    Q=R+W*L;  
    
  Te=(3/2)*(p/2)*Lm*(x(2)*x(3)-x(1)*x(4));
  %%% Matrix formulation of induction machine equations in common reference frame   
  %%% V= R*I + W*F + dF/dt = R*I + W*L*I +dF/dt = (R + W*L)*I + L*dI/dt      
  
   di=L\(V-Q*I);
   dwm=p/2*(Te-TL)/J-Bm/J*wm;
   dx=[di;dwm;wk]; % State equations
% ------------------------------------------------------------------