  % Induction motor line starting
% Choice of reference frame
% Currents as state variables
 function IMstartflux
 close all, clc
 warning off
 global L p Vs wo  km
 info= ...
        ['                         INDUCTION MOTOR STARTING                               '                                
         '                    (Fluxes and speed as state variables)                       '
         ' The motor is starting at no load and, after reaching steady-state speed, a step'   
         ' load torque is suddenly applied.                                               '  
         ' The dynamic equations are expressed in canonical first-order matrix form       ' 
         ' adopting fluxes and speed as state variables, and are solved using ODE45.      '
         ' The machine parameters may be changed by editing the file <IMstartflux.m>.     ' 
         ' The user has the choice of selecting one of three common (dq) reference frames:'
         ' STATOR, ROTOR or SYNCHRONOUS.                                                  '
         ' The torque, speed,and current responses are plotted.                           '];
                                            
      disp(info)
 %pause
 % --------------------  REFERENCE FRAME SELECTION  ----------------------------
      km=menu('Select a common reference frame','STATOR','SYNCHRONOUS','ROTOR','QUIT');
if km==4,clc,close all;return; end                
% ---------------------------  ODE  -------------------------------------------
 xo=[0 0 0 0 0 0]';        %initial conditions 
 tf=2 ;               % final time
 tspan=[0 tf] ; 
 options=odeset('RelTol',1e-3,'maxstep',1e-3); %,'outputfcn','odeplot','outputsel',5) ;
 [t,x]=ode45(@IMstf,tspan,xo,options);
 % --------------------  OUTPUT VARIABLES  --------------------------------------
 flux=x(:,1:4)'  ;
 cur=L\flux ;
 spd=x(:,5)*60/(p*pi);  %rpm
 phids=x(:,1); phiqs=x(:,2);
 ids=cur(1,:)' ;   iqs=cur(2,:)' ;
 Te=0.75*p*(iqs.*phids-ids.*phiqs);
 %ias=ids.*cos(x(:,6))-iqs.*sin(x(:,6));
 %ibs=ids.*sin(x(:,6))+iqs.*cos(x(:,6));
   psi=x(:,6);
  vas=Vs*cos(wo*t); vbs=Vs*sin(wo*t);
  vds=vas.*cos(psi)+vbs.*sin(psi);  % Voltage expressed in dq common reference frame
  vqs=-vas.*sin(psi)+vbs.*cos(psi);
  % --------------------  GRAPHICAL DISPLAYS  ----------------------------------
  figure('Units','Normalized','Position',[.02 .54 .5 .35],'Name','Induction motor starting transients');
  figure(1)
   plot(t,spd,'color',[.5 0 0]);title('Speed vs time');xlabel('Time [s]'),ylabel('Speed [rpm]'),grid
   figure('Units','Normalized','Position',[.02 .09 .5 .35],'Name','Induction motor starting transients');
  figure(2) 
   plot(0,0,'w',t,ids,'-r',t,iqs,'-g');title('Currents vs time','color','r');xlabel('Time [s]'),ylabel('ids &iqs [A]'),grid
    if km==1, ref='Stator frame';
     elseif km==2, ref='Synchronous frame';
     else ref='rotor frame';
     end
  legend(ref)
   figure('Units','Normalized','Position',[.52 .54 .45 .35],'Name','Induction motor starting transients');
  figure(3)
   plot(t,Te);title('Torque vs time');xlabel('Time [s]'),ylabel('Torque [N.m]'),grid
   figure('Units','Normalized','Position',[.52 .09 .45 .35],'Name','Induction motor starting transients');
  figure(4)
   plot(spd,Te);title('Torque vs Speed');xlabel('Speed [rpm]'),ylabel('Torque [N.m]'),grid
    figure('Units','Normalized','Position',[.35 .09 .4 .35],'Name','Induction motor starting transients');
  figure(5)
   plot(0,0,'w',t,phids,'-r',t,phiqs,'-g');title('Stator fluxes vs time');xlabel('Time [s]'),ylabel('phids & phiqs [Wb]'),grid 
   if km==1, ref='Stator frame';
     elseif km==2, ref='Synchronous frame';
     else ref='rotor frame';
   end
   legend(ref)
  figure(6);
  plot(t,vds,'-m',t,vqs,'-g');title('Voltage vs time','color','b');xlabel('Time [s]'),ylabel('Voltage [V]'),grid 
    if km==1, ref='Stator frame';
     elseif km==2, ref='Synchronous frame';
     else ref='rotor frame';
    end
  legend(ref)
 
 h0=figure('Units','Normalized','menubar','none','numbertitle','off','Position',[.87 .8 .1 .15]); 
 
      uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','clc,clear,close all,return', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.06 0.55 0.2], ...
	      'String','CLOSE', ...
	      'TooltipString','Close'); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit imstartflux', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.39 0.55 0.2], ...
	      'String','EDIT', ...
	      'TooltipString','Edit');
      uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','run imstartflux', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.7 0.55 0.2], ...
	      'String','RUN', ...
	      'TooltipString','Run');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dxdt=IMstf(t,x)
global L p Vs wo km
% --------------------     MACHINE PARAMETERS ----------------------
Vll=220    ; %Line-to-line rms voltage rating [V]
Rs=0.531    ; %Stator resistance (Ohm)
Rr=0.408   ; %Rotor resistance  (Ohm)
Lsl=2.5e-3 ; %Stator leakage inductance(H)
Lrl=2.5e-3    ; %Rotor leakage inductance  (H)
Lm=84.7e-3   ; %Magnetizing inductance  (H) 
freq0=60   ; %Base frequency (Hz)
wo=2*pi*freq0 ; %Base frequency (rad/s)
p=4        ; %number of poles
J=0.1    ; %Moment of inertia (kg.m^2)
TLf=15; %Load torque [N.m]
Bm=0.02; %Damping constant
% -------------------------INDUCTANCE MATRIX-------------------------
Ls=Lsl+Lm ; Lr=Lrl+Lm;

L=[Ls  0   Lm  0   
   0   Ls  0   Lm 
   Lm  0   Lr  0
   0   Lm  0   Lr];
  
% --------------------------STATE EQUATIONS-------------------------- 
 wm=x(5) ;
        if km==1,  wk=0;                  
             elseif km==2,  wk=wo;          
             elseif km==3, wk=wm;
        end
      if t<0.8, TL=0;else TL=TLf; end
 Vs=Vll*sqrt(2/3);
 psi=x(6);
 vas=Vs*cos(wo*t);    vbs=Vs*sin(wo*t);
 
 %vds=vas*cos(psi))+vbs*sin(psi) ;
 %vqs=-vas*sin(psi)+vbs*cos(psi) ;  
 
 vabs=[vas;vbs];
      Rotf=[cos(psi)  sin(psi)
          -sin(psi)  cos(psi) ];
 vdqs=Rotf*vabs;
 vds=vdqs(1); vqs=vdqs(2);
 
    V=[vds vqs 0 0]';
    F=[x(1) x(2) x(3) x(4)]';
    I=L\F;
     
   R=diag([Rs Rs Rr Rr]);
    
   W=[ 0        -wk         0        0
      wk         0          0        0
       0         0          0     -(wk-wm)
       0         0       (wk-wm)     0     ];
   
  Te=(3/2)*(p/2)*(I(2)*F(1)-I(1)*F(2));  
 %%% Matrix formulation of induction machine equations in common reference frame   
 %%% V= R*I + W*F + dF/dt   where I = Linv*F
   
dfdt=V-R*I-W*F;
%dfdt=V-(L\R+W)*F;
dwmdt=p/2*(Te-TL)/J-Bm/J*wm;
dxdt  =[dfdt;dwmdt;wk]; % State equations
% -------------------------------------------------------------------------
