% Steady-state characteristics of the induction machine under balanced operation
clc,clear
% --------------------     MACHINE PARAMETERS [pu]  ----------------------------
Vs=1.0   ; % Stator voltage 
gam=0;     % Phase (rad)
ws=1.0;    % Stator frequency
Rs=0.01  ; % Stator resistance 
Rr=0.02  ; % Rotor resistance  
Lsl=.10 ;  % Stator leakage inductance
Lrl=.10  ; % Rotor leakage inductance  
Lm=4.50;   % Magnetizing inductance
n1=0.0;    % Min speed
n2=1.0;    % Max speed
Ls=Lsl+Lm ; Lr=Lrl+Lm;
% ------------------------------------------------------------------------------
wk=ws;     %Synchronous frame
% NOTE: In steady-state balanced conditions, all variables are constant (dc)
%       when expressed in the synchronous frame.
wmm=n1:0.005:n2;                      % speed range 
l=length(wmm);
z=zeros(1,l);
spd=z;Te=z;Is=z;Ir=z;isd=z;Pelec=z;Pmec=z;
for m=1:l                             % # of steps
 wm=wmm(m);

 V=[Vs*cos(gam) Vs*sin(gam) 0 0]';    % Voltage vector

 R=diag([Rs Rs Rr Rr]);               % Resistance matrix   
       
 L=[Ls  0   Lm  0   
    0   Ls  0   Lm 
    Lm  0   Lr  0
    0   Lm  0   Lr];                  % Inductance matrix
    
    rot90=[0 -1;1 0];    O=zeros(2);        
    
  W=[wk*rot90       O                  % Frame speed matrix ...
        O    (wk-wm)*rot90];           % with respect to stator and rotor      
    
  Z=R+W*L;                             % Impedance (real) matrix
  
  I=Z\V;                               % Current vector
        
  spd(m)=wm;                             % Speed
  Te(m)=Lm*(I(2)*I(3)-I(1)*I(4));        % Electromagnetic torque (average)
  Is(m)=sqrt(I(1)^2+I(2)^2);             % Stator current (magnitude)
  Ir(m)=sqrt(I(3)^2+I(4)^2);             % Rotor current  (magnitude)
  isd(m)=I(1); isq(m)=I(2);              % dq components of Is
  Pelec(m)=V(1)*I(1)+V(2)*I(2);          % Electric power in
  Pmec(m)=(Lm*(I(2)*I(3)-I(1)*I(4)))*wm; % Mechanical power out
end
% --------------------------- PLOTS  -----------------------------------------------
 close all
 figure('Units','Normalized','Position',[.012 .5 .49 .4],'Name','Steady-state characteristics')
 figure(1)  
   plot(spd,Te,'color',[0.5 0 0],'linewidth',2),xlabel('SPEED [pu]'),ylabel('TORQUE [pu]'),grid;
 
 figure('Units','Normalized','Position',[.5 .5 .45 .4],'Name','Steady-state characteristics')
 figure(2)   
     plot(spd,Is,'-r',spd,Ir,'-b'),xlabel('SPEED [pu]'),ylabel('CURRENTS [pu]'),grid
     legend('Stator','Rotor',3)
     
 figure('Units','Normalized','Position',[.012 .05 .49 .35],'Name','Steady-state characteristics')
 figure(3)   
     plot(spd,Pelec,'-m',spd,Pmec,'-c','linewidth',2),xlabel('SPEED [pu]'),ylabel('POWERS [pu]'),grid
     legend('P_e_l_e_c','P_m_e_c',2)    

 figure('Units','Normalized','Position',[.5 .05 .45 .35],'Name','Steady-state characteristics')
 figure(4) 
     plot(isd,isq,'-r','linewidth',2),title('Stator current circle diagram'),xlabel('ids [pu]'),ylabel('iqs [pu]'),grid,hold on
     vecarrow([0 0],[Vs 0],'b')
     ll=fix(length(wmm)/1.1);
     vecarrow([isd(ll-3) isq(ll-3)],[isd(ll) isq(ll)],'r');
     text(Vs+.2,-.2,'V_s','color','b') 
     text(isd(ll)+.2,isq(ll),'\omega_m','color','r') 
     axis equal
     
   h0=figure('Units','Normalized','menubar','none','numbertitle','off','Position',[.85 .33 .1 .14]); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','close all,clc', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.32 0.55 0.15], ...
	      'String','CLOSE', ...
	      'TooltipString','Close'); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit imstatic1', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.56 0.5 0.15], ...
	      'String','EDIT', ...
	      'TooltipString','Edit');
      uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','run imstatic1', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.27 0.8 0.55 0.15], ...
	      'String','RERUN', ...
	      'TooltipString','Run');
      uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','close all,run immod1', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor','y', ...
 	      'Position',[0.07 0.09 0.88 0.17], ...
	      'String','TRANSIENTS', ...
	      'TooltipString','Trans');

% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    