% Three-phase reluctance motor 
% Phase variable model... 
% per unit
 clc,close all,clear
       msg1= ...
        ['                     RELUCTANCE MOTOR                       '
         '                     [Phase variable model]                 '
         '                (abc stationary reference frame)            '
         '                           [per unit]                       '
         ' A three-phase reluctance motor is started from standstill  ' 
         ' at rated voltage and a light load, due to the action of the' 
         ' damper windings. When the speed is close to synchronism,   '  
         ' the motor pulls into step due to the effect of saliency.   '                                   
         ' The load is then suddenly increased.                       '
         ' The dynamic equations are expressed in canonical first-    ' 
         ' order matrix form in the stationary phase variable model in'
         ' which the state variables are the stator currents ias, ibs,'
         ' ics; the d-axis and q-axis damper currents idk and iqk; the'
         ' angular speed wm; and the angle theta between the stator   '
         ' as-axis and the rotor direct axis.                         '                                                      
         ' Torque, speed, current, and torque angle responses are     '
         ' plotted.                                                   '];
                                      
     h0=msgbox(msg1,'Reluctance motor','help');
     set(h0,'Units','normalized','Resize','on','Position',[.32 .58 .34 .34])  
         
 % --------------------     MACHINE PARAMETERS [pu] -----------------
Ra=0.03    ; %Stator resistance 
Ld=1.80    ; %Direct-axis inductance
Lq=0.90    ; %Quadrature-axis inductance
Lal=0.10   ; %Armature leakage inductance
Rkd=0.2    ; %D-axis damper resistance
Rkq=0.2    ; %Q-axis damper resistance
Lkd=1.3    ; %D-axis damper inductance
Lkq=1.2    ; %Q-axis damper inductance
Lakd=1.0   ; %Peak mutual inductance between direct-axis damper and armature
Lakq=1.0   ; %Peak mutual inductance between quadrature-axis damper and armature
freq0=60   ; %Frequency [Hz]
wo=2*pi*freq0 ; %Base frequency (rad/s)
H=0.5       ; %Inertia constant [s]
TLi=0.10    ; %Initial load torque 
TLf=0.25    ; %Final load torque < T_max=.5*Vm^2*(1/Lq-1/Ld)
Bm=1e-3     ; %Frictional coefficient
Vm=1.0      ; %Stator voltage
t1=5.0      ; %Time of load application [s]
gam=2*pi/3  ; 
  waitbar(0,'Computing...');
     
%  ---------------------------  ODE  -------------------------------------------
 xo=[0 0 0 0 0 0 0]';          % initial conditions 
 tf=7.0;                       % final time
 tspan=[0 tf] ; 
 options=odeset('RelTol',1e-2,'maxstep',1e-3); 
 [t,x]=ode45(@SMabcrelf2,tspan,xo,options,Ra,Rkd,Rkq,Ld,Lq,Lal,Lkd,Lkq,Lakd,Lakq,Vm,wo,H,Bm,TLi,TLf,t1);
  
   % --------------------  OUTPUT VARIABLES ----------------------------------
 spd=x(:,6);  % speed
 ias=x(:,1)  ;  ibs=x(:,2) ; ics=x(:,3);       % Armature abc currents 
 ikd=x(:,4) ; ikq=x(:,5) ;                     % Damper currents  
 ang=180/pi*(wo*t-x(:,7));               
 angle=mod(ang,360)-270;                       % Torque angle delta [o]
 lg=length(spd); 
 rg=round(.38*lg):lg;
 
 Lad=Ld-Lal;   Laq=Lq-Lal;
 L2=(Lad-Laq)/2;
  
  for k=1:lg
     h1=waitbar(k/lg); 
    
    theta=x(k,7);               % Angle between stator as_axis and rotor direct axis  
    
    i=x(k,1:5);                 % Current vector
       
    dLss=-2*L2*[sin(2*theta)       sin(2*theta-gam)  sin(2*theta+gam)  
                  sin(2*theta-gam) sin(2*(theta-gam))    sin(2*theta)
                  sin(2*theta+gam)   sin(2*theta)    sin(2*(theta+gam))];  
         
    
    dLsr=[ -Lakd*sin(theta)      -Lakq*cos(theta)
           -Lakd*sin(theta-gam)  -Lakq*cos(theta-gam)
           -Lakq*sin(theta+gam)  -Lakq*cos(theta+gam)];
            
    dLdtheta=[dLss   dLsr
              dLsr'  zeros(2)];
        
    Te(k)=0.5*i*dLdtheta*i'  ;   % Electromagnetic torque  
    
 end
  close(h0,h1)
  
  % --------------------  GRAPHICAL DISPLAYS -----------------------------------
  figure('Units','Normalized','Position',[.01 .49 .45 .35],'Name','Reluctance motor');
  figure(1)
  plot(t,spd,'color',[0.5 0 0]);title('Reluctance motor start','color','r');xlabel('Time [s]'),ylabel('Speed [pu]'),grid
  
  figure('Units','Normalized','Position',[.46 .49 .45 .35],'Name','Reluctance motor');
  figure(2)
  plot(t,Te,'-m',[0 t1],[TLi TLi],'-c',[t1 tf],[TLf TLf],'-c',[0 tf],[0 0],'-k');title('Reluctance motor start','color','r');
        xlabel('Time [s]'),ylabel('Torque [pu]'),grid
        legend('Te', 'TL')       
           
  figure('Units','Normalized','Position',[.01 .08 .45 .32],'Name','Reluctance motor');
  figure(3) 
  plot(t,ias,'-r',t,ibs,'-g',t,ics,'b');title('Reluctance motor start','color','r');xlabel('Time [s]'),ylabel('Stator currents [pu]'),grid
  
  figure('Units','Normalized','Position',[.46 .08 .45 .32],'Name','Reluctance motor');
  figure(4) 
  plot(t,ikd,'-b',t,ikq,'-g');title('Reluctance motor start','color','r');xlabel('Time [s]'),ylabel('Damper currents [pu]'),grid 
  
  figure('Units','Normalized','Position',[.01 .52 .45 .35],'Name','Reluctance motor' );
  figure(5)
  plot(t(rg),angle(rg),'color',[0.5 0 0]);title('Reluctance motor start','color','r');xlabel('Time [s]'),ylabel('Torque angle [o]'),grid
  
  figure('Units','Normalized','Position',[.46 .52 .45 .35],'Name','Reluctance motor');
  figure(6)
  pt=fix(.705*lg); aa=[angle(pt) spd(pt)]; bb=[angle(pt+10) spd(pt+10)]; 
  plot(angle(rg),spd(rg),'color',[0.7 0.2 0]);title('Reluctance motor start','color','r');
    xlabel('Torque angle [o]'),ylabel('Speed [pu]'),grid,hold on,
    vecarrow(aa,bb,'r')
  hold off  
   
      h0=figure('Units','Normalized','menubar','none','numbertitle','off','Position',[.88 .50 .05 .1]); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','clc,close all,return', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.1 0.6 0.22], ...
	      'String','CLOSE', ...
	      'TooltipString','Close'); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','smabcrel2', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.31 0.6 .22], ...
	      'String','RERUN', ...
	      'TooltipString','Run');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit smabcrelf2', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.55 0.6 .22], ...
	      'String','STATE', ...
	      'TooltipString','State');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit smabcrel2', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.77 .6 0.22], ...
	      'String','PARAM', ...
	      'TooltipString','Parameters');
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
